@@ -8,7 +8,7 @@ from datetime import timedelta
 from enum import Enum
 from random import random
 from time import sleep
-from typing import Union, Any, Iterable
+from typing import Union, Any, Iterable, NamedTuple
 
 import rx
 from rx import operators as ops, Observable
@@ -24,6 +24,15 @@ from influxdb_client.client.write.retry import WritesRetry
 logger = logging.getLogger(__name__)
 
 
+try:
+    import dataclasses
+    from dataclasses import dataclass
+
+    _HAS_DATACLASS = True
+except ModuleNotFoundError:
+    _HAS_DATACLASS = False
+
+
 class WriteType(Enum):
     """Configuration which type of writes will client use."""
 
@@ -70,8 +79,14 @@ class WriteOptions(object):
         self.exponential_base = exponential_base
         self.write_scheduler = write_scheduler
 
-    def to_retry_strategy(self):
-        """Create a Retry strategy from write options."""
+    def to_retry_strategy(self, **kwargs):
+        """
+        Create a Retry strategy from write options.
+
+        :key retry_callback: The callable ``callback`` to run after retryable error occurred.
+                             The callable must accept one argument:
+                                - `Exception`: an retryable error
+        """
         return WritesRetry(
             total=self.max_retries,
             retry_interval=self.retry_interval / 1_000,
@@ -79,6 +94,7 @@ class WriteOptions(object):
             max_retry_delay=self.max_retry_delay / 1_000,
             max_retry_time=self.max_retry_time / 1_000,
             exponential_base=self.exponential_base,
+            retry_callback=kwargs.get("retry_callback", None),
             method_whitelist=["POST"])
 
     def __getstate__(self):
@@ -126,18 +142,6 @@ class PointSettings(object):
         self.defaultTags[key] = self._get_value(value)
 
 
-class _BatchItem(object):
-    def __init__(self, key, data, size=1) -> None:
-        self.key = key
-        self.data = data
-        self.size = size
-        pass
-
-    def __str__(self) -> str:
-        return '_BatchItem[key:\'{}\', size: \'{}\']' \
-            .format(str(self.key), str(self.size))
-
-
 class _BatchItemKey(object):
     def __init__(self, bucket, org, precision=DEFAULT_WRITE_PRECISION) -> None:
         self.bucket = bucket
@@ -157,8 +161,23 @@ class _BatchItemKey(object):
             .format(str(self.bucket), str(self.org), str(self.precision))
 
 
+class _BatchItem(object):
+    def __init__(self, key: _BatchItemKey, data, size=1) -> None:
+        self.key = key
+        self.data = data
+        self.size = size
+        pass
+
+    def to_key_tuple(self) -> (str, str, str):
+        return self.key.bucket, self.key.org, self.key.precision
+
+    def __str__(self) -> str:
+        return '_BatchItem[key:\'{}\', size: \'{}\']' \
+            .format(str(self.key), str(self.size))
+
+
 class _BatchResponse(object):
-    def __init__(self, data: _BatchItem, exception=None):
+    def __init__(self, data: _BatchItem, exception: Exception = None):
         self.data = data
         self.exception = exception
         pass
@@ -173,15 +192,63 @@ def _body_reduce(batch_items):
 
 
 class WriteApi:
-    """Implementation for '/api/v2/write' endpoint."""
+    """
+    Implementation for '/api/v2/write' endpoint.
 
-    def __init__(self, influxdb_client, write_options: WriteOptions = WriteOptions(),
-                 point_settings: PointSettings = PointSettings()) -> None:
-        """Initialize defaults."""
+    Example:
+        .. code-block:: python
+
+            from influxdb_client import InfluxDBClient
+            from influxdb_client.client.write_api import SYNCHRONOUS
+
+
+            # Initialize SYNCHRONOUS instance of WriteApi
+            with InfluxDBClient(url="http://localhost:8086", token="my-token", org="my-org") as client:
+                write_api = client.write_api(write_options=SYNCHRONOUS)
+    """
+
+    def __init__(self,
+                 influxdb_client,
+                 write_options: WriteOptions = WriteOptions(),
+                 point_settings: PointSettings = PointSettings(),
+                 **kwargs) -> None:
+        """
+        Initialize defaults.
+
+        :param influxdb_client: with default settings (organization)
+        :param write_options: write api configuration
+        :param point_settings: settings to store default tags.
+        :key success_callback: The callable ``callback`` to run after successfully writen a batch.
+
+                               The callable must accept two arguments:
+                                    - `Tuple`: ``(bucket, organization, precision)``
+                                    - `str`: written data
+
+                               **[batching mode]**
+        :key error_callback: The callable ``callback`` to run after unsuccessfully writen a batch.
+
+                             The callable must accept three arguments:
+                                - `Tuple`: ``(bucket, organization, precision)``
+                                - `str`: written data
+                                - `Exception`: an occurred error
+
+                             **[batching mode]**
+        :key retry_callback: The callable ``callback`` to run after retryable error occurred.
+
+                             The callable must accept three arguments:
+                                - `Tuple`: ``(bucket, organization, precision)``
+                                - `str`: written data
+                                - `Exception`: an retryable error
+
+                             **[batching mode]**
+        """
         self._influxdb_client = influxdb_client
         self._write_service = WriteService(influxdb_client.api_client)
         self._write_options = write_options
         self._point_settings = point_settings
+        self._success_callback = kwargs.get('success_callback', None)
+        self._error_callback = kwargs.get('error_callback', None)
+        self._retry_callback = kwargs.get('retry_callback', None)
 
         if influxdb_client.default_tags:
             for key, value in influxdb_client.default_tags.items():
@@ -217,22 +284,71 @@ class WriteApi:
     def write(self, bucket: str, org: str = None,
               record: Union[
                   str, Iterable['str'], Point, Iterable['Point'], dict, Iterable['dict'], bytes, Iterable['bytes'],
-                  Observable] = None,
+                  Observable, NamedTuple, Iterable['NamedTuple'], 'dataclass', Iterable['dataclass']
+              ] = None,
               write_precision: WritePrecision = DEFAULT_WRITE_PRECISION, **kwargs) -> Any:
         """
         Write time-series data into InfluxDB.
 
+        :param str bucket: specifies the destination bucket for writes (required)
         :param str, Organization org: specifies the destination organization for writes;
                                       take the ID, Name or Organization;
                                       if it's not specified then is used default from client.org.
-        :param str bucket: specifies the destination bucket for writes (required)
         :param WritePrecision write_precision: specifies the precision for the unix timestamps within
                                                the body line-protocol. The precision specified on a Point has precedes
                                                and is use for write.
-        :param record: Points, line protocol, Pandas DataFrame, RxPY Observable to write
-        :key data_frame_measurement_name: name of measurement for writing Pandas DataFrame
-        :key data_frame_tag_columns: list of DataFrame columns which are tags, rest columns will be fields
-        """
+        :param record: Point, Line Protocol, Dictionary, NamedTuple, Data Classes, Pandas DataFrame or
+                       RxPY Observable to write
+        :key data_frame_measurement_name: name of measurement for writing Pandas DataFrame - ``DataFrame``
+        :key data_frame_tag_columns: list of DataFrame columns which are tags,
+                                     rest columns will be fields - ``DataFrame``
+        :key record_measurement_key: key of record with specified measurement -
+                                     ``dictionary``, ``NamedTuple``, ``dataclass``
+        :key record_measurement_name: static measurement name - ``dictionary``, ``NamedTuple``, ``dataclass``
+        :key record_time_key: key of record with specified timestamp - ``dictionary``, ``NamedTuple``, ``dataclass``
+        :key record_tag_keys: list of record keys to use as a tag - ``dictionary``, ``NamedTuple``, ``dataclass``
+        :key record_field_keys: list of record keys to use as a field  - ``dictionary``, ``NamedTuple``, ``dataclass``
+
+        Example:
+            .. code-block:: python
+
+                # Record as Line Protocol
+                write_api.write("my-bucket", "my-org", "h2o_feet,location=us-west level=125i 1")
+
+                # Record as Dictionary
+                dictionary = {
+                    "measurement": "h2o_feet",
+                    "tags": {"location": "us-west"},
+                    "fields": {"level": 125},
+                    "time": 1
+                }
+                write_api.write("my-bucket", "my-org", dictionary)
+
+                # Record as Point
+                from influxdb_client import Point
+                point = Point("h2o_feet").tag("location", "us-west").field("level", 125).time(1)
+                write_api.write("my-bucket", "my-org", point)
+
+        DataFrame:
+            The index of `Pandas DataFrame <https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.html>`_
+            is used as a ``timestamp`` for written data. The index should be `PeriodIndex <https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.PeriodIndex.html#pandas.PeriodIndex>`_
+            or its must be transformable to ``datetime`` by
+            `pandas.to_datetime <https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.to_datetime.html#pandas.to_datetime>`_.
+
+            If you would like to transform a column to ``PeriodIndex``, you can use something like:
+
+            .. code-block:: python
+
+                import pandas as pd
+
+                # DataFrame
+                data_frame = ...
+                # Set column as Index
+                data_frame.set_index('column_name', inplace=True)
+                # Transform index to PeriodIndex
+                data_frame.index = pd.to_datetime(data_frame.index, unit='s')
+
+        """  # noqa: E501
         org = get_org_query_param(org=org, client=self._influxdb_client)
 
         if self._point_settings.defaultTags and record is not None:
@@ -309,12 +425,16 @@ class WriteApi:
             self._serialize(record.to_line_protocol(), record.write_precision, payload, **kwargs)
 
         elif isinstance(record, dict):
-            self._serialize(Point.from_dict(record, write_precision=write_precision),
+            self._serialize(Point.from_dict(record, write_precision=write_precision, **kwargs),
                             write_precision, payload, **kwargs)
         elif 'DataFrame' in type(record).__name__:
             serializer = DataframeSerializer(record, self._point_settings, write_precision, **kwargs)
             self._serialize(serializer.serialize(), write_precision, payload, **kwargs)
-
+        elif hasattr(record, "_asdict"):
+            # noinspection PyProtectedMember
+            self._serialize(record._asdict(), write_precision, payload, **kwargs)
+        elif _HAS_DATACLASS and dataclasses.is_dataclass(record):
+            self._serialize(dataclasses.asdict(record), write_precision, payload, **kwargs)
         elif isinstance(record, Iterable):
             for item in record:
                 self._serialize(item, write_precision, payload, **kwargs)
@@ -334,7 +454,7 @@ class WriteApi:
             self._write_batching(bucket, org, data.to_line_protocol(), data.write_precision, **kwargs)
 
         elif isinstance(data, dict):
-            self._write_batching(bucket, org, Point.from_dict(data, write_precision=precision),
+            self._write_batching(bucket, org, Point.from_dict(data, write_precision=precision, **kwargs),
                                  precision, **kwargs)
 
         elif 'DataFrame' in type(data).__name__:
@@ -344,6 +464,12 @@ class WriteApi:
                 self._write_batching(bucket, org,
                                      serializer.serialize(chunk_idx),
                                      precision, **kwargs)
+        elif hasattr(data, "_asdict"):
+            # noinspection PyProtectedMember
+            self._write_batching(bucket, org, data._asdict(), precision, **kwargs)
+
+        elif _HAS_DATACLASS and dataclasses.is_dataclass(data):
+            self._write_batching(bucket, org, dataclasses.asdict(data), precision, **kwargs)
 
         elif isinstance(data, Iterable):
             for item in data:
@@ -371,7 +497,13 @@ class WriteApi:
 
         logger.debug("Write time series data into InfluxDB: %s", batch_item)
 
-        retry = self._write_options.to_retry_strategy()
+        if self._retry_callback:
+            def _retry_callback_delegate(exception):
+                return self._retry_callback(batch_item.to_key_tuple(), batch_item.data, exception)
+        else:
+            _retry_callback_delegate = None
+
+        retry = self._write_options.to_retry_strategy(retry_callback=_retry_callback_delegate)
 
         self._post_write(False, batch_item.key.bucket, batch_item.key.org, batch_item.data,
                          batch_item.key.precision, urlopen_kw={'retries': retry})
@@ -402,12 +534,15 @@ class WriteApi:
     def _jitter_delay(self):
         return timedelta(milliseconds=random() * self._write_options.jitter_interval)
 
-    @staticmethod
-    def _on_next(response: _BatchResponse):
+    def _on_next(self, response: _BatchResponse):
         if response.exception:
             logger.error("The batch item wasn't processed successfully because: %s", response.exception)
+            if self._error_callback:
+                self._error_callback(response.data.to_key_tuple(), response.data.data, response.exception)
         else:
             logger.debug("The batch item: %s was processed successfully.", response)
+            if self._success_callback:
+                self._success_callback(response.data.to_key_tuple(), response.data.data)
 
     @staticmethod
     def _on_error(ex):
@@ -430,4 +565,9 @@ class WriteApi:
         """Set your object with the provided dict."""
         self.__dict__.update(state)
         # Init Rx
-        self.__init__(self._influxdb_client, self._write_options, self._point_settings)
+        self.__init__(self._influxdb_client,
+                      self._write_options,
+                      self._point_settings,
+                      success_callback=self._success_callback,
+                      error_callback=self._error_callback,
+                      retry_callback=self._retry_callback)
